######################################################################
#
# File: 	txChild.py
#
# Purpose:	This forks a child process that
#		comes with filters for listening for special commands
#
# Version:	$Id: txChild.py,v 1.6 2001/04/20 22:21:26 pstoltz Exp $
#
# Copyright 2000 Tech-X Corporation, all rights reserved
#
######################################################################

import thread, pty, os, time, sys, termios, TERMIOS
##########
#
# This is a class to provide a child process with a built-in
# filter on the output pipe.
#
##########
class txChild:

  # This is the constructor method.   
  # It stores the output stream as member data.
  # 
  # **Arguments**
  #
  # *proc_name* a string containing the name of the child process
  #
  # *args* the argument list obtained from argv
  #
  # **returns** none
  def __init__(self,proc_name,args):
    self.newlock = thread.allocate_lock()
    self.child_proc=proc_name
    self.spawnChild(args)

  # This is a method to spawn a child process.   
  # It uses the pseudo-terminal module to fork a process.
  # Turning off ECHO is important to avoid multiple writes to the screen.
  #
  # **Arguments**
  #
  # *args* the argument list obtained from argv
  #
  # **returns** none
  def spawnChild(self,args):
    # txChild needs a fork 
    (self.pid, self.fd) = pty.fork()
  
    # This is the parent process, so set up pipes 
    if self.pid:         
      self.istr = os.fdopen(self.fd, 'r')
      self.ostr = os.fdopen(self.fd, 'w')

    # This is the child process 
    #
    else:
    # Turn off echoing on stdin of the child to prevent second echo
      tios = termios.tcgetattr(0)
      tios[3] = tios[3] & ~TERMIOS.ECHO
      tios = termios.tcsetattr(0, TERMIOS.TCSANOW, tios)
      os.execv("/usr/bin/env", ("/usr/bin/env", self.child_proc, args))

    thread.start_new_thread(self.transferIstr, (self.istr,self.newlock) )
    #Sleep to ensure it starts up
    time.sleep(2.5)

  # This function listens to output of the child process, filters it, 
  # and prints to the screen if appropriate.  In this base class, it
  # has no functionality and should be over-written in the derived class.
  #
  # **Arguments** none
  #
  # **returns** none
  def transferIstr(self):
    pass

